from spaceone.api.identity.v1 import project_pb2, project_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class Project(BaseAPI, project_pb2_grpc.ProjectServicer):

    pb2 = project_pb2
    pb2_grpc = project_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectService', metadata) as project_svc:
            return self.locator.get_info('ProjectInfo', project_svc.create(params))

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectService', metadata) as project_svc:
            return self.locator.get_info('ProjectInfo', project_svc.update(params))

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectService', metadata) as project_svc:
            project_svc.delete(params)
            return self.locator.get_info('EmptyInfo')

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectService', metadata) as project_svc:
            return self.locator.get_info('ProjectInfo', project_svc.get(params))

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectService', metadata) as project_svc:
            project_vos, total_count, project_groups_info = project_svc.list(params)
            return self.locator.get_info('ProjectsInfo', project_vos, total_count,
                                         minimal=self.get_minimal(params), project_groups_info=project_groups_info)

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectService', metadata) as project_svc:
            return self.locator.get_info('StatisticsInfo', project_svc.stat(params))

    def add_member(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectService', metadata) as project_svc:
            return self.locator.get_info('ProjectRoleBindingInfo', project_svc.add_member(params))

    def modify_member(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectService', metadata) as project_svc:
            return self.locator.get_info('ProjectRoleBindingInfo', project_svc.modify_member(params))

    def remove_member(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectService', metadata) as project_svc:
            project_svc.remove_member(params)
            return self.locator.get_info('EmptyInfo')

    def list_members(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('ProjectService', metadata) as project_svc:
            project_map_vos, total_count = project_svc.list_members(params)
            return self.locator.get_info('ProjectRoleBindingsInfo', project_map_vos, total_count,
                                         minimal=self.get_minimal(params))



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel
from spaceone.identity.model.user_model import User


class APIKey(MongoModel):
    api_key_id = StringField(max_length=40, generate_id='api-key', unique=True)
    state = StringField(max_length=20, default='ENABLED', choices=('ENABLED', 'DISABLED'))
    user_id = StringField(max_length=40, required=True)
    user = ReferenceField('User', null=True, default=None, reverse_delete_rule=CASCADE)
    domain_id = StringField(max_length=40, required=True)
    last_accessed_at = DateTimeField(default=None, null=True)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        'updatable_fields': [
            'state',
            'last_accessed_at'
        ],
        'minimal_fields': [
            'api_key_id',
            'state',
            'user_id'
        ],
        'ordering': ['api_key_id'],
        'indexes': [
            'state',
            'user_id',
            'domain_id',
            'last_accessed_at'
        ]
    }



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel


class ProjectGroup(MongoModel):
    project_group_id = StringField(max_length=40, generate_id='pg', unique=True)
    name = StringField(max_length=40)
    parent_project_group = ReferenceField('self', null=True, default=None, reverse_delete_rule=DENY)
    parent_project_group_id = StringField(max_length=40, null=True, default=None)
    tags = DictField()
    domain_id = StringField(max_length=255)
    created_by = StringField(max_length=255, null=True)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        'updatable_fields': [
            'name',
            'parent_project_group',
            'parent_project_group_id',
            'tags'
        ],
        'minimal_fields': [
            'project_group_id',
            'name'
        ],
        'change_query_keys': {
            'user_project_groups': 'project_group_id'
        },
        'reference_query_keys': {
            'parent_project_group': 'self'
        },
        'ordering': ['name'],
        'indexes': [
            # 'project_group_id',
            'parent_project_group',
            'parent_project_group_id',
            'domain_id',
        ]
    }



from spaceone.identity.manager.api_key_manager import APIKeyManager
from spaceone.identity.manager.authorization_manager import AuthorizationManager
from spaceone.identity.manager.domain_manager import DomainManager
from spaceone.identity.manager.domain_secret_manager import DomainSecretManager
from spaceone.identity.manager.provider_manager import ProviderManager
from spaceone.identity.manager.service_account_manager import ServiceAccountManager
from spaceone.identity.manager.policy_manager import PolicyManager
from spaceone.identity.manager.project_group_manager import ProjectGroupManager
from spaceone.identity.manager.project_manager import ProjectManager
from spaceone.identity.manager.user_manager import UserManager
from spaceone.identity.manager.role_manager import RoleManager
from spaceone.identity.manager.role_binding_manager import RoleBindingManager
from spaceone.identity.manager.domain_owner_manager import DomainOwnerManager
from spaceone.identity.manager.token_manager.local_token_manager import LocalTokenManager
from spaceone.identity.manager.token_manager.external_token_manager import ExternalTokenManager
from spaceone.identity.manager.token_manager.domain_owner_token_manager import DomainOwnerTokenManager
from spaceone.identity.manager.endpoint_manager import EndpointManager
from spaceone.identity.manager.email_manager import EmailManager



from spaceone.api.identity.v1 import domain_pb2, domain_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class Domain(BaseAPI, domain_pb2_grpc.DomainServicer):

    pb2 = domain_pb2
    pb2_grpc = domain_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainService', metadata) as domain_svc:
            data = domain_svc.create(params)
            return self.locator.get_info('DomainInfo', data)

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainService', metadata) as domain_svc:
            data = domain_svc.update(params)
            return self.locator.get_info('DomainInfo', data)

    def change_auth_plugin(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainService', metadata) as domain_svc:
            data = domain_svc.change_auth_plugin(params)
            return self.locator.get_info('DomainInfo', data)

    def update_plugin(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainService', metadata) as domain_svc:
            data = domain_svc.update_plugin(params)
            return self.locator.get_info('DomainInfo', data)

    def verify_plugin(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainService', metadata) as domain_svc:
            data = domain_svc.verify_plugin(params)
            return self.locator.get_info('EmptyInfo')


    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainService', metadata) as domain_svc:
            domain_svc.delete(params)
            return self.locator.get_info('EmptyInfo')

    def enable(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainService', metadata) as domain_svc:
            data = domain_svc.enable(params)
            return self.locator.get_info('DomainInfo', data)

    def disable(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainService', metadata) as domain_svc:
            data = domain_svc.disable(params)
            return self.locator.get_info('DomainInfo', data)

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainService', metadata) as domain_svc:
            data = domain_svc.get(params)
            return self.locator.get_info('DomainInfo', data)

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainService', metadata) as domain_svc:
            data, total_count = domain_svc.list(params)
            return self.locator.get_info('DomainsInfo', data, total_count, minimal=self.get_minimal(params))

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainService', metadata) as domain_svc:
            return self.locator.get_info('StatisticsInfo', domain_svc.stat(params))

    def get_public_key(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainService', metadata) as domain_svc:
            data = domain_svc.get_public_key(params)
            return self.locator.get_info('DomainPublicKeyInfo', data['pub_jwk'], data['domain_id'])



import logging

from spaceone.core import config
from spaceone.core.manager import BaseManager

_LOGGER = logging.getLogger(__name__)

ENDPOINT_MAP = {
    'public': 'ENDPOINTS',
    'internal': 'INTERNAL_ENDPOINTS'
}


class EndpointManager(BaseManager):

    def list_endpoints(self, query={}, endpoint_type='public'):
        endpoint_map = ENDPOINT_MAP.get(endpoint_type, 'ENDPOINTS')
        endpoints = config.get_global(endpoint_map, [])
        return endpoints, len(endpoints)



import logging
import time
import random
from abc import abstractmethod, ABC, ABCMeta

from spaceone.core import config, utils, cache
from spaceone.core.manager import BaseManager
from spaceone.core.auth.jwt.jwt_util import JWTUtil

from spaceone.identity.error.error_authentication import *


__all__ = ['TokenManager', 'JWTManager']
_LOGGER = logging.getLogger(__name__)


class TokenManager(BaseManager, ABC):

    is_authenticated = False

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._load_conf()

    @abstractmethod
    def issue_temporary_token(self, **kwargs):
        pass

    @abstractmethod
    def issue_token(self, **kwargs):
        pass

    @abstractmethod
    def refresh_token(self, user_id, domain_id, **kwargs):
        pass

    @abstractmethod
    def authenticate(self, user_id, domain_id, credentials):
        pass

    @abstractmethod
    def check_refreshable(self, key, ttl):
        pass

    def _load_conf(self):
        identity_conf = config.get_global('IDENTITY') or {}
        token_conf = identity_conf.get('token', {})
        self.CONST_TOKEN_TIMEOUT = token_conf.get('token_timeout', 1800)
        self.CONST_VERIFY_CODE_TIMEOUT = token_conf.get('verify_code_timeout', 3600)
        self.CONST_REFRESH_TIMEOUT = token_conf.get('refresh_timeout', 3600)
        self.CONST_REFRESH_TTL = token_conf.get('refresh_ttl', -1)
        self.CONST_REFRESH_ONCE = token_conf.get('refresh_once', True)


class JWTManager(TokenManager, metaclass=ABCMeta):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.old_refresh_key = None

    def issue_temporary_token(self, **kwargs):
        raise NotImplementedError('TokenManager.issue_temporary_token not implemented!')

    def issue_token(self, **kwargs):
        raise NotImplementedError('TokenManager.issue_token not implemented!')

    def refresh_token(self, user_id, domain_id, **kwargs):
        raise NotImplementedError('TokenManager.refresh_token not implemented!')

    def authenticate(self, user_id, domain_id, credentials):
        raise NotImplementedError('TokenManager.authenticate not implemented!')

    def check_refreshable(self, refresh_key, ttl):
        if self.CONST_REFRESH_ONCE:
            if cache.is_set() and cache.get(f'refresh-token:{refresh_key}') is None:
                raise ERROR_INVALID_REFRESH_TOKEN()

        if ttl == 0:
            raise ERROR_REFRESH_COUNT()

        self.is_authenticated = True
        self.old_refresh_key = refresh_key

    def issue_access_token(self, user_type, user_id, domain_id, **kwargs):
        private_jwk = self._get_private_jwk(kwargs)
        permissions = kwargs.get('permissions')
        timeout = kwargs.get('timeout')
        if timeout is None:
            timeout = self.CONST_TOKEN_TIMEOUT

        payload = {
            'cat': 'ACCESS_TOKEN',
            'user_type': user_type,
            'did': domain_id,
            'aud': user_id,
            'iat': int(time.time()),
            'exp': int(time.time() + timeout)
        }

        if permissions:
            payload['permissions'] = permissions

        encoded = JWTUtil.encode(payload, private_jwk)
        return encoded

    def issue_refresh_token(self, user_type, user_id, domain_id, **kwargs):
        refresh_private_jwk = self._get_refresh_private_jwk(kwargs)
        ttl = kwargs.get('ttl')
        timeout = kwargs.get('timeout')

        if ttl is None:
            ttl = self.CONST_REFRESH_TTL
        elif ttl > self.CONST_REFRESH_TTL:
            raise ERROR_MAX_REFRESH_COUNT(max_refresh_count=self.CONST_REFRESH_TTL)

        if timeout is None:
            timeout = self.CONST_REFRESH_TIMEOUT

        refresh_key = self._generate_refresh_key()

        payload = {
            'cat': 'REFRESH_TOKEN',
            'user_type': user_type,
            'did': domain_id,
            'aud': user_id,
            'iat': int(time.time()),
            'exp': int(time.time() + timeout),
            'key': refresh_key,
            'ttl': ttl
        }

        encoded = JWTUtil.encode(payload, refresh_private_jwk)

        if self.CONST_REFRESH_ONCE:
            self._set_refresh_token_cache(refresh_key)

        return encoded

    @staticmethod
    def _generate_refresh_key():
        return utils.random_string(16)

    @staticmethod
    def _get_private_jwk(kwargs):
        if 'private_jwk' not in kwargs:
            raise ERROR_NOT_FOUND_PRIVATE_KEY(purpose='Access Token')

        return kwargs['private_jwk']

    @staticmethod
    def _get_refresh_private_jwk(kwargs):
        if 'refresh_private_jwk' not in kwargs:
            raise ERROR_NOT_FOUND_PRIVATE_KEY(purpose='Refresh Token')

        return kwargs['refresh_private_jwk']

    def _set_refresh_token_cache(self, new_refresh_key):
        if cache.is_set():
            if self.old_refresh_key:
                cache.delete(f'refresh-token:{self.old_refresh_key}')

            cache.set(f'refresh-token:{new_refresh_key}', '', expire=self.CONST_REFRESH_TIMEOUT)

    def create_verify_code(self, user_id, domain_id):
        if cache.is_set():
            verify_code = self._generate_verify_code()
            cache.delete(f'verify-code:{domain_id}:{user_id}')
            cache.set(f'verify-code:{domain_id}:{user_id}', verify_code, expire=self.CONST_VERIFY_CODE_TIMEOUT)
            return verify_code

    @staticmethod
    def check_verify_code(user_id, domain_id, verify_code):
        if cache.is_set():
            cached_verify_code = cache.get(f'verify-code:{domain_id}:{user_id}')
            if cached_verify_code == verify_code:
                return True
        return False

    @staticmethod
    def _generate_verify_code():
        return str(random.randint(100000, 999999))



from spaceone.api.identity.v1 import authorization_pb2, authorization_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class Authorization(BaseAPI, authorization_pb2_grpc.AuthorizationServicer):

    pb2 = authorization_pb2
    pb2_grpc = authorization_pb2_grpc

    def verify(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('AuthorizationService', metadata) as auth_service:
            auth_data = auth_service.verify(params)
            return self.locator.get_info('AuthorizationResponse', auth_data)

